require 'yandex/disk'

module CarrierWave
  module Storage
    module Yandex
      # Rubocop, shut up
      class Disk < Abstract
        # Stubs we must implement to create and save
        # files (here on Yandex.Disk)

        # Store a single file
        def helper_create_publish(file)
          location = "/#{uploader.store_path}"
          location_path = "/#{uploader.store_dir}"

          yandex_disk_client.mkdir_p(location_path)

          yandex_disk_client.put!(file.path.to_s, location)

          yandex_disk_client.make_public(location)
        end

        def helper_update_model(make_public_res)
          location = "/#{uploader.store_path}"

          public_url = ERB::Util.url_encode make_public_res[:public_url]

          file_id = { location: location, public_url: public_url }.to_json

          uploader.model.update_column uploader.mounted_as, file_id
        end

        def store!(file)
          # I had to break this method into 2 helpers just because
          # rubocop insisted on that. Isn't it stupid?
          res = helper_create_publish(file)

          helper_update_model(res)
        end

        # Retrieve a single file
        def retrieve!(file_id)
          file_id_hash = (JSON.parse file_id.gsub('=>', ':')).symbolize_keys

          public_url = file_id_hash[:public_url]

          location = file_id_hash[:location]

          CarrierWave::Storage::Yandex::Disk::File
            .new(public_url, location, yandex_disk_client)
        end

        def yandex_disk_client
          @yandex_disk_client ||= begin
            ::Yandex::Disk::Client.new(access_token: config[:access_token])
          end
        end

        private

        def config
          @config ||= {}

          @config[:access_token] ||= uploader.yandex_disk_access_token

          @config
        end
        # Rubocop, shut up
        class File
          # alike http://www.rubydoc.info/gems/carrierwave/CarrierWave/Storage/Fog/File
          include CarrierWave::Utilities::Uri

          def initialize(public_url, location, yandex_disk_client)
            @public_url = public_url
            @location = location
            @yandex_disk_client = yandex_disk_client
          end

          def filename
            ::File.basename @location
          end

          def extension
            ::File.extname @location
          end

          def url
            query = 'https://cloud-api.yandex.net/v1/disk/public/resources/' \
            "download?public_key=#{@public_url}"
            uri = URI query
            res = Net::HTTP.get(uri)
            hash = JSON.parse(res)
            hash['href']
          end

          def public_url
            CGI.unescape @public_url
          end

          def storage_path
            # deviation from http://www.rubydoc.info/gems/carrierwave/CarrierWave/Storage/Fog/File
            @location
          end

          def delete
            @yandex_disk_client.delete @location
            @yandex_disk_client.delete(::File.dirname(@location))
          end
        end
      end
    end
  end
end
